<?php
/*
 * Copyright (C) 2007 Niek Linnenbank <niek.linnenbank@planet.nl>
 */

/* Make sure we are not directly invoked */
if(!defined("INDEX"))
	die();

/* Include required classes */
include_once("include/Table.class.php");
include_once("include/Util.class.php");

/**
 * Abstraction for dynamic lists.
 * This class is an abstraction layer for dynamic lists in HTML forms.
 */
abstract class DynamicList {

	protected $items = array();
	protected $identifier;
	protected $title;
	protected $width;
	
	/** An message saying the list is empty. */
	protected $emptyListMessage;
	/** An message saying the item already exists. */
	protected $alreadyExistsMessage;
	
	/**
	 * Class constructor.
	 * The class constructor.
	 * @param identifier The unique identifier of the list.
	 * @param title The title of the dynamic list which the final Table will have.
	 * @see Table
	 */
	public function DynamicList($identifier, $title = null, $width = 500) {
		$this->identifier = $identifier;
		$this->title      = $title;
		$this->width      = intval($width);
	}
	
	/**
	 * Fill in the list using HTTP POST.
	 * This function will fill in the System using variables from $_POST.
	 * @return Always returns true.
	 */
	public function fillWithPost() {

		/* Reset the $_POST array */
		reset($_POST);
	
		/* Now fill us with the values from $_POST */
		while(list($key, $value) = each($_POST)) {
			if(ereg("^" . $this->identifier, $key))
				$this->addItem($key, $value);
		}

		/* See if the user submitted a new item */
		if(isset($_POST['create' . $this->identifier]) && !empty($_POST['new' . $this->identifier])) {
			/* First see if the given item value doesn't already exist in the current item array */
			if(array_search($_POST['new' . $this->identifier], $this->items) !== false ||
				/* Or if the new item already exists in the database */
				$this->databaseItemExists(null, $_POST['new' . $this->identifier]) != null)
			{
				Output::htmlError($this->alreadyExistsMessage, "Error", false);
				
			/* Find an empty key and add the new item */
			} else {
				$find = 0;
				while(array_key_exists($this->identifier . $find, $this->items))
					$find++;
				
				/* Now add the new item */
				$this->items[$this->identifier . $find] = $_POST['new' . $this->identifier];
			}
		
		}
		
		/* See if the user pressed 'Delete' for any of the items */
		reset($_POST);
		while(list($key, $value) = each($_POST)) {
			/* Is this the submitted value for one of the delete buttons? */
			if(ereg("^delete" . $this->identifier, $key) && !empty($value)) {
				/* Remove the item if we have a match */
				$itemId = substr($key, strlen($this->identifier) + strlen("Delete"));
				unset($this->items[$this->identifier . $itemId]);
			}
		}
	
		/* Success */
		return true;
	}
	
	/**
	 * Fill in the list using the database.
	 * This function is responsible for filling in the list using data from the database.
	 * @return boolean indicating success or failure.
	 * @see Database
	 */
	abstract public function fillWithDatabase();
	
	
	/**
	 * Add an item.
	 * This function adds an item.
	 * @key key The key of the new item.
	 * @param value The value of the new item.
	 */
	public function addItem($key, $value) {
		if(array_key_exists($key, $this->items))
			return;
		else
			$this->items[$key] = $value;
	}
	
	/**
	 * Remove an item.
	 * This function removes an item.
	 * @param key The key of the item to be deleted.
	 */
	public function deleteItem($key) {
	}

	/**
	 * Save the list.
	 * This function will call insert() or update() to save the list.
	 * @return Boolean true or false indicating success of failure.
	 */
	public function save() {
		/* We need to keep track of all saved id's, so they won't be DELETE'ed */
		$savedIds = array();
		
		/* First reset the items pointer */
		reset($this->items);
		
		/* Loop all items */
		while(list($key, $value) = each($this->items)) {
		
			/* Fetch the itemId */
			$itemId = substr($key, strlen($this->identifier));

			/* See if this item already exists in the database, searching by it's ID number */
			if($this->databaseItemExists($itemId, null) != null) {
				/* The ID already exists. Check if the offered value is already in the database (for another itemId!) */
				if($this->databaseItemExists($itemId, $value) != null)
				{
					Output::htmlError($this->alreadyExistsMessage, "Error", false);
				/* It doesn't exist, so we update our item with the new value */
				} else
					$this->databaseUpdateItem($itemId, $value);
			
			/* The ID doesn't exist yet */
			} else {
				/* see if the given value already exists. */
				if($this->databaseItemExists(null, $value) != null) {
					Output::htmlError($this->alreadyExistsMessage, "Error", false);
				
				/* Item doesn't exist yet, so we INSERT it */
				} else {
					$this->databaseInsertItem($value);
					$itemId = $this->databaseItemExists(null, $value);
				}
			}
			/* Push this itemId to the savedIds list, so it won't be DELETE'ed afterwards */
			if(!empty($itemId))
				array_push($savedIds, $itemId);
		}
		/* Now remove all items which are not on the savedIds list */
		$this->databaseDeleteItemsExcluding($savedIds);
		
		/* Return */
		return !Output::haveErrors();
	}

	/**
	 * Checks if the given item exist.
	 * Checks if the given item already exists in the database.
	 * @param itemId The itemId to search for.
	 * @param value The item value to search for.
	 * @returns Boolean true or false.
	 */
	abstract public function databaseItemExists($itemId, $value);

	/**
	 * Insert a new item.
	 * This function will insert a new item with value $value in the database.
	 * @param value The value to be inserted.
	 */
	abstract protected function databaseInsertItem($value);

	/**
	 * Update an existing item.
	 * This function will update an existing item in the database with $newValue.
	 * @param itemId The id number of the item.
	 * @param newValue The new value of the item.
	 */	
	abstract protected function databaseUpdateItem($itemId, $newValue);
	
	/**
	 * Delete existing item(s).
	 * This function is responsible for removing item(s) from the database.
	 * @param idArray An array containing id's which NOT to delete.
	 */
	abstract protected function databaseDeleteItemsExcluding($idArray);
	
	/**
	 * Checks if the list is empty.
	 * Returns true if the list is empty.
	 * @return Boolean true or false.
	 */
	public function isEmpty() {
		return (count($this->items) <= 0) ? true : false;
	}
	
	/**
	 * Output the list in HTML.
	 * This function will output the list in a HTML table.
	 */
	public function html() {
	
		/*
		echo "this->items<br>";
		print_r($this->items);
		echo "<br><br>";
		echo "$_POST<br>";
		print_r($_POST);
		echo "<br><br>";*/
	
		/* Create a new HTML table */
		if($this->title != null)
			$table = new Table($this->width);
		else
			$table = new Table($this->width, "center", "normaltable", false);
			
		$table->addColumn($this->title, "*");
		$table->addColumn("");
	
		/* Reset the items array pointer */
		reset($this->items);
	
		/* Loop all items and output <input> for each */
		while(list($key, $value) = each($this->items)) {
			$table->addRow("<input type=\"text\" name=\"" . Util::makeFormSafe($key) . "\" value=\"" . Util::makeFormSafe($value) . "\" size=\"40\">");
			$table->addRow("<input type=\"submit\" name=\"delete" . Util::makeFormSafe($key) . "\" value=\"Delete\" onClick=\"return verifyDelete('" . Util::makeJavascriptSafe($value) . "')\">");
		}
		
		/* If the table is empty, show this to the user */
		if($table->isEmpty()) {
			$table->addRow($this->emptyListMessage, "100", "center", "middle");
			$table->addRow("");
		}
		
		/* Add an empty line, and a create input field to let the user add new items */
		$table->addRow("<br><input type=\"text\" name=\"new" . $this->identifier . "\" value=\"\" size=\"40\"><br><br>");
		$table->addRow("<br><input type=\"submit\" name=\"create" . $this->identifier . "\" value=\"Create\"><br><br>");
		
		/* Return the final HTML table */
		return $table->html();
	}

}
?>
