<?php
/*
 * Copyright (C) 2007 Niek Linnenbank <niek.linnenbank@planet.nl>
 */

/* Make sure we are not directly invoked */
if(!defined("INDEX"))
	die();

/* Include required classes */
include_once("include/Field.class.php");
include_once("include/Table.class.php");
include_once("include/VersionList.class.php");
include_once("include/Util.class.php");

/**
 * Abstraction for operating systems.
 * This class is an abstraction layer for the os and version table in the MySQL database.
 */
class OS {

	private $fields   = array();
	private $versions = null;
	private $exists   = null;
	
	/**
	 * Class constructor
	 * @param id The id number of the operating system. This can be null for new operating systems.
	 */
	public function OS($id) {
		$this->addField("id", "ID", "text", intval($id));
		$this->addField("name", "Operating System Name", "text");
		
		/* We need a list of versions too for this operating system */
		$this->versions = new VersionList($this->getField("id")->getValue());
	}
	
	/**
	 * Fill in the operating system using HTTP POST.
	 * This function will fill in the operating system using variables from $_POST.
	 * @return Always returns true.
	 */
	public function fillWithPost() {
		/* Now fill us with the values from $_POST */
		$this->fill($_POST);
		$this->versions->fillWithPost();
		/* Success */
		return true;
	}
	
	/**
	 * Fill in the operating system using the database.
	 * This function will fill in all fields using data from the database.
	 * @see Database
	 */
	public function fillWithDatabase() {
		/* Now we query the database for all the information about this particular operating system */
		$result = Database::queryToArray("SELECT id, name FROM os WHERE id='" . Util::makeSqlSafe($this->getField("id")->getValue()) . "'");

		/* Does the OS exists? */
		if(!$result) {
			Output::htmlError("Sorry, the operating system you are looking for, does not exist!");
			return false;
		} else {
			/* Fill in the operating system with database data */
			$this->fill($result);
			$this->versions->fillWithDatabase();
			/* Success */
			return true;
		}
	}
	
	/**
	 * Fill the operating system using values from an array.
	 * This fill fill in the operating system looking for key/values inside an array, and create it's VersionList.
	 * @see Field
	 * @see VersionList
	 */
	private function fill($array) {
		/* Loop all fields, and look for matching keys in $array */
		for($i = 0; $i < count($this->fields); $i++) {
			$field = $this->fields[$i];
		
			/* Does the key exist in $array? */
			if(isset($array[$field->getName()])) {
				$field->setValue($array[$field->getName()]);
			}
		}
	}
	
	/**
	 * Get a field.
	 * This function retrieves a Field from the operating system.
	 * @param name The name of the field.
	 * @see Field
	 */
	public function getField($name) {
		/* Loop all fields */
		for($i = 0; $i < count($this->fields); $i++) {
			/* Is the current field the one we are looking for? Return it if so. */
			if($this->fields[$i]->getName() == $name)
				return $this->fields[$i];
		}
		/* Not found! */
		return null;
	}
	
	/**
	 * Add a field.
	 * This will add a new field to the operating system.
	 * @param name The name of the field.
	 * @param description A descriptive text for the field.
	 * @param type The HTML type of the field.
	 * @param value Initial value of the field.
	 * @param width The width the field will have in HTML.
	 */
	public function addField($name, $description, $type, $value = "", $width = "20") {
		array_push($this->fields, new Field($name, $description, $type, $value, $width));
	}
	
	/**
	 * Checks if the operating system already exists.
	 * Returns true if the operating system already exists in the database.
	 * @return boolean true or false indicating if the operating system already exists.
	 */
	public function exists() {
		
		/* Do we already know if it exists? */
		if($this->exists === null) {
		
			/* No, query the database. */
			$result = Database::query("SELECT id FROM os WHERE id='" . Util::makeSqlSafe($this->getField("id")->getValue()) . "'"); 

			/* Update the exist field */	
			$this->exists =  (mysql_num_rows($result) > 0);
		}
		return $this->exists;
	}

	
	/**
	 * Save the operating system in the database.
	 * This will either INSERT or UPDATE the operating system in the database.
	 * @see Database
	 */
	public function save($overwrite) {

		/* Check if he/she filled in the operating system name, and complain otherwise */
		if($this->getField("name")->getValue() == "") {
			Output::htmlError("The operating system name is required. Please fill in a operating system name.", "Error", false);
			return false;
		}
		/* Does there already exist a system named like us? */
		$result = Database::query("SELECT id FROM os WHERE name='" . Util::makeSqlSafe($this->getField("name")->getValue()) . "' " .
					  "AND id!='" . Util::makeSqlSafe($this->getField("id")->getValue()) . "'"); 

		/* If there already exists a operating system named $name, bail out with an error. */
		if((mysql_num_rows($result) > 0)) {
			Output::htmlError("There already exists an operating system named &quot;" . Util::makeFormSafe($this->getField("name")->getValue()) . "&quot;. " .
					  "Please choose a different operating name.", "Error", false);
			return false;

		/* There must be at least one version */			
		} else if($this->versions->isEmpty()) {
			Output::htmlError("There must be at least one version available for an operating system.", "Error", false);
			return false;
		
		/* Now we may save the operating system in the database. */
		} else {		
			/* Execute an INSERT if we cannot overwrite. */
			if(!$overwrite) {
				Database::query("INSERT INTO os (name) VALUES ('" . Util::makeSqlSafe($this->getField("name")->getValue()) . "')");
				
				/* Read the new osId */
				$os = Database::queryToArray("SELECT id FROM os WHERE name='" . Util::makeSqlSafe($this->getField("name")->getValue()) . "'", false);
				$this->versions->setOsId($os[0]);
				
			/* Execute a UPDATE query on this system if it already exists. */			
			} else {
				Database::query("UPDATE os SET name='" . Util::makeSqlSafe($this->getField("name")->getValue()) . 
					"' WHERE id='" . Util::makeSqlSafe($this->getField("id")->getValue()) . "'");
			}
			return $this->versions->save();
		}
		/* Success */
		return true;
	}
	
	/**
	 * Remove the operating system from the database.
	 * This function will DELETE the operating system from the database.
	 * @return Always boolean true.
	 * @see Database
	 */
	public function delete() {

		/* Update all system entries which have any of this operating system's version */
                Database::query("UPDATE system SET version=NULL WHERE version IN (SELECT id FROM version WHERE os='" . Util::makeSqlSafe($this->getField("id")->getValue()) . "')");
                /* Delete all the versions of this operating system */
                Database::query("DELETE FROM version WHERE os='" . Util::makeSqlSafe($this->getField("id")->getValue()) . "'");
                /* And finally, delete the operating system entry itself */
                Database::query("DELETE FROM os WHERE id='" . Util::makeSqlSafe($this->getField("id")->getValue()) . "'");

		/* Success */		
		return true;
	}
	
	/**
	 * Output the operating system as an HTML form.
	 * This function will output the operating system as HTML form attributes, inside a Table.
	 */
	public function html() {
	
		/* Create a table to show the system details, with two columns. */
		$detailTable = new Table("650", "center", "normaltable", false);
		$detailTable->setTitle("Operating System Details");
		$detailTable->addColumn("", 200, "20");
		$detailTable->addColumn("", "*", "20");
		
		/* Now we fill in all our fields in $detailTable */
		for($i = 0; $i < count($this->fields); $i++) {
		
			/* We do not want the ID field to be shown in the form. */
			if($this->fields[$i]->getName() != "id") {
				$detailTable->addRow($this->fields[$i]->getLabel() . ":", 40, "left");
				$detailTable->addRow($this->fields[$i]->html(), 40, "left");
			}
		}
		
		/* Also output the VersionList */
		$detailTable->addRow("<br>Versions:", 40, "left", "top");
		$detailTable->addRow("<br>" . $this->versions->html() . "<br>", "*", "left");

		/* Output the final form, including the detailTable */
		return $detailTable->html();
	}
}
?>
