<?php
/*
 * Copyright (C) 2007 Niek Linnenbank <niek.linnenbank@planet.nl>
 */

/* Make sure we are not directly invoked */
if(!defined("INDEX"))
	die();

/* Include required classes */
include_once("include/Field.class.php");
include_once("include/Table.class.php");
include_once("include/Util.class.php");

/**
 * Abstraction for systems.
 * This class is an abstraction layer for the system table in the MySQL database.
 */
class System {

	private $fields = array();
	private $exists = null;
	
	/**
	 * Class constructor
	 * @param id The id number of the system. This can be null for new systems.
	 */
	public function System($id) {
		$this->addField("id", "ID", "text", intval($id));
		$this->addField("name", "System Name", "text");
		$this->addField("group", "Group", "combobox");
		$this->addField("hardware", "Hardware", "textarea", null, "7");
		$this->addField("user", "Administrated by", "combobox");
		$this->addField("os", "Operating System", "combobox");
		$this->addField("version", "Version", "combobox");
		$this->addField("special", "Specialities", "combobox");
		$this->addField("host", "Host IP", "text");
		$this->addField("ilo", "ILO IP", "text");
		$this->addField("dec", "DEC Server IP", "text");
		$this->addField("decport", "DEC Server Port", "text");
		$this->addField("console", "Console Switch IP", "text");
		$this->addField("comments", "Comments", "textarea", null, "7");
	}
	
	/**
	 * Fill in the system using HTTP POST.
	 * This function will fill in the System using variables from $_POST.
	 * @return Always returns true.
	 */
	public function fillWithPost() {
		/* Now fill us with the values from $_POST */
		$this->fill($_POST);
		/* Success */
		return true;
	}
	
	/**
	 * Fill in the system using the database.
	 * This function will fill in all fields using data from the database.
	 * @see Database
	 */
	public function fillWithDatabase() {
		/* Now we query the database for all the information about this particular system */
		$result = Database::queryToArray("SELECT	s.*,"									.
						 "		(SELECT v.os FROM version v WHERE v.id=s.version) as os "       	.
						 "FROM		system s "								.
						 "WHERE		s.id=" . Util::makeSqlSafe($this->getField("id")->getValue()));
		/* Does the system exists? */
		if(!$result) {
			Output::htmlError("Sorry, the system you are looking for, does not exist!");
			return false;
		} else {
			/* Fill in the system with database data */
			$this->fill($result);
			/* Success */
			return true;
		}
	}
	
	/**
	 * Fill the System using values from an array.
	 * This fill fill in the System looking for key/values inside an array, and create ComboBoxes where appropriate.
	 * @see Field
	 * @see ComboBox
	 */
	private function fill($array) {
	
		/* Loop all fields, and look for matching keys in $array */
		for($i = 0; $i < count($this->fields); $i++) {
			$field = $this->fields[$i];
		
			/* Does the key exist in $array? */
			if(isset($array[$field->getName()])) {
				$field->setValue($array[$field->getName()]);
			}
		}
		
		/* Query the database for a full list of groups, users, operating systems, versions and specials, in order to create a ComboBox. */
		$groups   = Database::queryToArray("SELECT id,name FROM `group` ORDER BY name", true);
		$users	  = Database::queryToArray("SELECT id,name FROM user ORDER BY name", true);
		$oss	  = Database::queryToArray("SELECT id,name FROM os ORDER BY name", true);
		$versions = Database::queryToArray("SELECT id,name FROM version WHERE os='" . Util::makeSqlSafe($this->getField("os")->getValue()) . "' ORDER BY name", true);
		$specials = Database::queryToArray("SELECT id,name FROM special ORDER BY name", true);

		/* If no group is set, use the default group */
		if($this->getField("group")->getValue() == "") {
			$defaultGroup = Database::queryToArray("SELECT id FROM `group` WHERE isdefault=true LIMIT 1", false);
			$this->getField("group")->setValue($defaultGroup[0]);
		}
		
		/* Now we create a ComboBox for the appropriate fields. */
		
		/* Group */
		$this->getField("group")->setHtml(ComboBox::createFromArray("group", $this->getField("group")->getValue(), $groups, false));
		/* User */
		$this->getField("user")->setHtml(ComboBox::createFromArray("user", $this->getField("user")->getValue(),	$users, true));
		/* OS */
		$this->getField("os")->setHtml(ComboBox::createFromArray("os", $this->getField("os")->getValue(), $oss, true));
		/* Version */
		$this->getField("version")->setHtml(ComboBox::createFromArray("version", $this->getField("version")->getValue(), $versions, true));
		/* Special */
		$this->getField("special")->setHtml(ComboBox::createFromArray("special", $this->getField("special")->getValue(), $specials, true));
	}
	
	/**
	 * Get a field.
	 * This function retrieves a Field from the system.
	 * @param name The name of the field.
	 * @see Field
	 */
	public function getField($name) {
		/* Loop all fields */
		for($i = 0; $i < count($this->fields); $i++) {
			/* Is the current field the one we are looking for? Return it if so. */
			if($this->fields[$i]->getName() == $name)
				return $this->fields[$i];
		}
		/* Not found! */
		return null;
	}
	
	/**
	 * Add a field.
	 * This will add a new field to the system.
	 * @param name The name of the field.
	 * @param description A descriptive text for the field.
	 * @param type The HTML type of the field.
	 * @param value Initial value of the field.
	 * @param width The width the field will have in HTML.
	 */
	public function addField($name, $description, $type, $value = "", $width = "20") {
		array_push($this->fields, new Field($name, $description, $type, $value, $width));
	}
	
	/**
	 * Checks if the System already exists.
	 * Returns true if the System already exists in the database.
	 * @return boolean true or false indicating if the system already exists.
	 */
	public function exists() {
		
		/* Do we already know if it exists? */
		if($this->exists === null) {
		
			/* No, query the database. */
			$result = Database::query("SELECT id FROM system WHERE id='" . Util::makeSqlSafe($this->getField("id")->getValue()) . "'"); 

			/* Update the exist field */	
			$this->exists =  (mysql_num_rows($result) > 0);
		}
		return $this->exists;
	}

	
	/**
	 * Save the system in the database.
	 * This will either INSERT or UPDATE the system in the database.
	 * @see Database
	 */
	public function save($overwrite) {

		/* Check if he/she filled in the system name, and complain otherwise */
		if($this->getField("name")->getValue() == "") {
			Output::htmlError("The system name is required. Please fill in a system name.", "Error", false);
			return false;
		
		/* Check if he/she filled in the group, and complain otherwise */
		} else if($this->getField("group")->getValue() == "") {
			Output::htmlError("The system group is required. Please choose a group.", "Error", false);
			return false;
			
		/* Check if the operating system if filled in, but not it's version */
		} else if($this->getField("os")->getValue() != "" && $this->getField("version")->getValue() == "") {
			Output::htmlError("You must also fill in a version for an operating system", "Error", false);
			return false;
		}
		
		/* Does there already exist a system named like us? */
		$result = Database::query("SELECT id FROM system WHERE name='" . Util::makeSqlSafe($this->getField("name")->getValue()) . "' " .
					  "AND id!='" . Util::makeSqlSafe($this->getField("id")->getValue()) . "'"); 

		/* If there already exists a system named $name, and we cannot overwrite, bail out with an error. */
		if((mysql_num_rows($result) > 0)) {
			Output::htmlError("There already exists a system named &quot;" . Util::makeFormSafe($this->getField("name")->getValue()) . "&quot;. " .
					  "Please choose a different name.", "Error", false);
			return false;
		} else {
			/* Make sure the Host IP address field contains only numbers and dots */
			
			$this->getField("host")->setValue(	preg_replace('/[^0-9.]/i', '', $this->getField("host")->getValue() ));
			
			/* ILO */
			if($_POST['terminal'] == "ilo") {
				$this->getField("ilo")->setValue(	preg_replace('/[^0-9.]/i', '', $this->getField("ilo")->getValue()  ));
				$this->getField("dec")->setValue("");
				$this->getField("decport")->setValue("");
				$this->getField("console")->setValue("");
			/* DEC */
			} else if ($_POST['terminal'] == "dec") {
				$this->getField("ilo")->setValue("");
				$this->getField("dec")->setValue(	preg_replace('/[^0-9.]/i', '', $this->getField("dec")->getValue()  ));
				$this->getField("decport")->setValue(	intval($this->getField("decport")->getValue()			   ));
				$this->getField("console")->setValue("");
			/* Console */
			} else if ($_POST['terminal'] == "console") {
				$this->getField("ilo")->setValue("");
				$this->getField("dec")->setValue("");
				$this->getField("decport")->setValue("");
				$this->getField("console")->setValue(preg_replace('/[^0-9.]/i', '', $this->getField("console")->getValue() ));
			/* None */
			} else {
				$this->getField("ilo")->setValue("");
				$this->getField("dec")->setValue("");
				$this->getField("decport")->setValue("");
				$this->getField("console")->setValue("");
			}
		
			/* Execute an INSERT if we cannot overwrite. */
			if(!$overwrite) {
				Database::query("INSERT INTO system (name, `group`, hardware, version, user, special, host, ilo, `dec`, decport, console, comments) " .
						"VALUES ('"	. Util::makeSqlSafe($this->getField("name")->getValue())	. "', "	.
						"	 '"	. Util::makeSqlSafe($this->getField("group")->getValue())	. "', " .
						"	 '"	. Util::makeSqlSafe($this->getField("hardware")->getValue())	. "', " .
						"	 '"	. Util::makeSqlSafe($this->getField("version")->getValue())	. "', " .
						"	 '"	. Util::makeSqlSafe($this->getField("user")->getValue())	. "', " .
						"	 '"	. Util::makeSqlSafe($this->getField("special")->getValue())	. "', " .
						"	 '"	. Util::makeSqlSafe($this->getField("host")->getValue())	. "', " .
						"	 '"	. Util::makeSqlSafe($this->getField("ilo")->getValue())		. "', " .
						"        '"	. Util::makeSqlSafe($this->getField("dec")->getValue())		. "', " .
						"        '"	. Util::makeSqlSafe($this->getField("decport")->getValue())	. "', " .
						"	 '"	. Util::makeSqlSafe($this->getField("console")->getValue())	. "', " .
						"	 '"	. Util::makeSqlSafe($this->getField("comments")->getValue())	. "')");

			/* Execute a UPDATE query on this system if it already exists. */			
			} else {
				Database::query( "UPDATE system "						  				.
						 "SET	name='" 	. Util::makeSqlSafe($this->getField("name")->getValue())	. "',"	.
						 "	`group`='"	. Util::makeSqlSafe($this->getField("group")->getValue())	. "',"	.
						 "	hardware='"	. Util::makeSqlSafe($this->getField("hardware")->getValue())	. "',"	.
						 "	user='"		. Util::makeSqlSafe($this->getField("user")->getValue())	. "',"	.
						 "	version='"	. Util::makeSqlSafe($this->getField("version")->getValue())	. "',"	.
						 "	special='"	. Util::makeSqlSafe($this->getField("special")->getValue())	. "',"	.
						 "	host='"		. Util::makeSqlSafe($this->getField("host")->getValue())	. "',"	.
						 "	ilo='"		. Util::makeSqlSafe($this->getField("ilo")->getValue())		. "',"	.
						 "	`dec`='"	. Util::makeSqlSafe($this->getField("dec")->getValue())		. "',"	.
						 "	decport='"	. Util::makeSqlSafe($this->getField("decport")->getValue())	. "',"	.
						 "	console='"	. Util::makeSqlSafe($this->getField("console")->getValue())	. "',"	.
						 "	comments='"	. Util::makeSqlSafe($this->getField("comments")->getValue())	. "' "	.
						 "WHERE	id='"		. Util::makeSqlSafe($this->getField("id")->getValue())		. "'");
			}
		}
		/* Success */
		return true;
	}
	
	/**
	 * Remove the system from the database.
	 * This function will DELETE the system from the database.
	 * @return Always boolean true.
	 * @see Database
	 */
	public function delete() {
		/* Execute a DELETE query on this system */
		Database::query("DELETE FROM system WHERE id='" . Util::makeSqlSafe($this->getField("id")->getValue()) . "'");
		/* Success */		
		return true;
	}
	
	/**
	 * Output the system as an HTML form.
	 * This function will output the system as HTML form attributes, inside a Table.
	 */
	public function html() {
	
		/* Keep track of the currently set terminal type */
		$terminalTypeSet = false;
	
		/* Create a table to show the system details, with two columns. */
		$detailTable = new Table("500", "center", "normaltable", false);
		$detailTable->setTitle("System Details");
		$detailTable->addColumn("", 200, "20");
		$detailTable->addColumn("", "*", "20");
		
		/* Now we fill in all our fields in $detailTable */
		for($i = 0; $i < count($this->fields); $i++) {
		
			/* We do not want the ID field to be shown in the form. */
			if($this->fields[$i]->getName() != "id") {

				$label = "";
				
				/* ILO, DEC and Console have a radio button */
				if($this->fields[$i]->getName() == "ilo" || $this->fields[$i]->getName() == "dec" || $this->fields[$i]->getName() == "decport" ||
					$this->fields[$i]->getName() == "console")
				{
					$label = "<table border=\"0\" cellpadding=\"0\" cellspacing=\"0\">"	.
						 "<tr><td width=\"30\" align=\"center\">";
				
					if($this->fields[$i]->getName() != "decport") {
						$label .= "<input type=\"radio\" name=\"terminal\" value=\"" . $this->fields[$i]->getName() . "\"";
						
						/* Are we the current value? */
						if(!empty($_POST['terminal']) && $this->fields[$i]->getName() == $_POST['terminal']) {
							$label .= " checked=\"checked\"";
							$terminalTypeSet = true;
					
						/* No terminal type set, are we filled in? */
						} else if(strlen($this->fields[$i]->getValue()) > 0) {
							$label .= " checked=\"checked\"";
							$terminalTypeSet = true;
						}
						$label .= ">";
					}
					$label .= "</td><td width=\"*\"  align=\"left\">" . $this->fields[$i]->getLabel() . ":</td>"		.
						  "</tr></table>";
				} else
					$label = $this->fields[$i]->getLabel() . ":";

				$detailTable->addRow($label, 40, "left");
				$detailTable->addRow($this->fields[$i]->html(), 40, "left");
			}
		}

		/* Output the final form, including the detailTable */
		return $detailTable->html();
	}
}
?>
